# -*- coding: utf-8 -*-
from urlparse import urljoin

from django.conf import settings
from django.db.models.fields.files import ImageFieldFile
from django.forms import FileInput, TextInput, CharField, ImageField, \
                         DateField, DateTimeField
from django.utils.safestring import mark_safe
from django.contrib.admin.templatetags.adminmedia import admin_media_prefix
from django.contrib.admin.widgets import ForeignKeyRawIdWidget, \
                                         AdminTextareaWidget, \
                                         AdminTextInputWidget, \
                                         AdminDateWidget, \
                                         AdminSplitDateTime


class PathWidget(ForeignKeyRawIdWidget):
    """Path widget like raw_id on django-admin."""
    def __init__(self, attrs=None):
        from tcms.models import Page
        rel = Page._meta.get_field('path').rel
        super(PathWidget, self).__init__(rel, attrs)

    def render(self, *args, **kwargs):
        return mark_safe(super(PathWidget, self).render(*args, **kwargs)\
                                                .replace('href="../../../',
                                                         'href="/admin/'))


class ImageFileInput(FileInput):
    """File input with direct link to view/download file"""
    def render(self, name, value, attrs=None):
        """Render file input and add preview image and link to file on
        MEDIA_URL path if value is given"""
        out = super(ImageFileInput, self).render(name, value, attrs=attrs)
        if value and isinstance(value, (str, unicode, ImageFieldFile)):
            if isinstance(value, ImageFieldFile):
                file_url = value.url
            else:
                file_url = urljoin(settings.MEDIA_URL, value)
            out += """<a class="file-preview" target="_blank" href="%s">
                   <img height="100" src="%s" /></a>""" % (file_url, file_url)
        return mark_safe(out)


class RelatedWidget(TextInput):
    """Related values widget"""
    def __init__(self, related_link, *args, **kwargs):
        """Related Widget with raw_id behavior. @related_link is url to open"""
        self.related_link = related_link
        super(RelatedWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        """Renders input field and adds a custom related link."""
        attrs = attrs or {}
        if not attrs.has_key('class'):
            attrs['class'] = 'vForeignKeyRawIdAdminField vLargeTextField'
        out = super(RelatedWidget, self).render(name, value, attrs=attrs) + \
        """<a href="%s" class="related-lookup" id="lookup_id_%s"
              onclick="return showRelatedObjectLookupPopup(this);">
           <img src="%simg/admin/selector-search.gif" width="16" height="16"
                alt="Lookup" /></a>""" % \
               (self.related_link, name, admin_media_prefix())
        return mark_safe(out)


class RichEditorWidget(AdminTextareaWidget):
    """Textarea widget with link to enable rich editing"""
    def render(self, name, value, attrs=None):
        """Renders input field and adds a custom related link."""
        attrs = attrs or {}
        attrs['class'] = 'vLargeTextField'
        out = super(RichEditorWidget, self).render(name, value, attrs=attrs) + \
        '<a href="#" class="richedit" id="text_id_%s">Rich editor</a>' % name
        return mark_safe(out)


# Fields
class AdminCharField(CharField):
    """Charfield using Admin text input widget"""
    widget = AdminTextInputWidget


class AdminTexareaField(CharField):
    """Charfield using Admin textarea widget"""
    widget = AdminTextareaWidget


class RichTexareaField(CharField):
    """Charfield with rich-editor support"""
    widget = RichEditorWidget(attrs={'class': 'vLargeTextField richtext'})


class PreviewImageField(ImageField):
    """Imagefile with small preview linked image"""
    widget = ImageFileInput


class AdminDateField(DateField):
    """Date field with admin widget"""
    widget = AdminDateWidget


class AdminDateTimeField(DateTimeField):
    """Date field with admin widget"""
    widget = AdminSplitDateTime



# -*- coding: utf-8 -*-
from django.conf import settings

from tcms.models import Page, CMSID
from tcms.utils import id_from_cache


def cms(request):
    """
    Loads needed context data to render a CMS page.

    Raw values will be used if running in an instance with admin option
    enabled, if not, rendered content will be used instead.

    The processor looks for a cmsid GET parameter, if preset will load the
    page with that id.

    This function uses a cache to speed up page id retrieving and avoid
    unnecesary database hits. Will fallback to tipocms app if there's no
    id for url/locate pair.

    Request object might have a @cms_url attribute which will be used as an
    override for current request path when looking for pages. If it's a list,
    then each path is tested until the first match and finally current path
    is tested.

    If request.no_cms is True then everything is skipped and no CMS data is
    loaded.
    """
    if getattr(request, 'no_cms', False):
        return {}

    is_admin = 'django.contrib.admin' in settings.INSTALLED_APPS

    if is_admin and request.GET.get(CMSID):
        cmsid = request.GET.get(CMSID)
    else:
        paths = getattr(request, 'cms_url', None)
        # use path override if preset, it can be a list defining several
        # possible paths that can be used with current page
        if not paths:
            paths = [request.path]
        elif not isinstance(paths, list):
            paths = [paths]
        if request.path not in paths:
            paths.insert(0, request.path)

        cmsid = id_from_cache(paths, getattr(request, 'LANGUAGE_CODE', None))

    ctx = {'cms': None}
    if cmsid is not None:
        try:
            ctx['cms'] = Page.objects.select_related('path').get(pk=cmsid)
        except Page.DoesNotExist: # shouldn't happen
            pass
        else:
            ctx['cms'].load(rendered=not is_admin)
    return ctx



# -*- coding: utf-8 -*-
"""
Page template data types definitions. Here are defined custom data types
with it's corresponding behavior to interact with the CMS. Project models
can be added and will work in a raw_id way.
"""
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.forms import ChoiceField, BooleanField
from django.utils.safestring import mark_safe
from django.db.models.loading import get_model

from tcms.utils import save_b64_image, image_to_b64
from tcms.fields import RelatedWidget, AdminCharField, PreviewImageField, \
                        RichTexareaField, AdminDateField, AdminDateTimeField


__all__ = ['BaseType', 'PlainType', 'RawIdType', 'Text', 'BigText', 'Option',
           'Flag', 'Image', 'Date', 'DateTime', 'BASE_TYPES']

IMAGES_UPLOAD_TO = getattr(settings, 'TCMS_IMAGES_UPLOAD_TO', 'cms/image/%Y/%m/%d')


class BaseType(object):
    """Base type, types should extend this class"""
    description = '' # type description
    field = AdminCharField # form field, an AdminCharField by default

    def as_field(self, *args, **kwargs):
        """Return field for current type, an AdminCharField by default."""
        return self.field(*args, **kwargs)

    def to_database(self, form, name):
        """Prepare data in form to be saved into database. Override in
        subclass
        """
        raise NotImplementedError, 'implement in subclass'

    def form_value(self, value):
        """Return treated value for usage in form"""
        return self.value(value)

    def value(self, value):
        """Return value to be shown.
            * return value marked safe if it's an string.
            * return passed value in other cases
        """
        if isinstance(value, (str, unicode)):
            value = mark_safe(value)
        return value

    def to_xml(self, value):
        """Return value to be included in a XML file. Must return a dict
        with data to be include. Return value by default.
        """
        return {'value': value}

    def from_xml(self, data):
        """Return value that was stored in a XML file. @data should be
        a dictionary with same format as returned by to_xml method.
        """
        if isinstance(data, dict):
            return data.get('value')

    @classmethod
    def name(cls):
        """Return type name, works as an instance or class method"""
        try:
            if issubclass(cls, BaseType):
                return cls.__name__.lower()
        except TypeError:
            return cls.__class__.__name__.lower()

    @classmethod
    def __str__(cls):
        return cls.description


class PlainType(BaseType):
    """Plain text data type."""
    def to_database(self, form, name):
        """Prepares value to database storage, raises ValueError if value is
        missing"""
        if name not in form.cleaned_data:
            raise ValueError, 'missing value'
        return form.cleaned_data[name]


class RawIdType(PlainType):
    """Raw Id type, allows related definition to proyect models entries"""
    # model or url must be defined or AttributeError will raised in that case
    model = None  # model related too
    url = None    # url override in case url cannot be built from model like
                  # wsmodels/Product
    model_key = 'pk'  # Custom model primary key field
    select_related = None  # Select related option, it *must* be a list

    def __init__(self, *args, **kwargs):
        self.model = kwargs.pop('model', self.model)
        self.url = kwargs.pop('url', self.url)
        self.model_key = kwargs.pop('model_key', self.model_key)
        self.select_related = kwargs.pop('select_related', self.select_related)
        super(RawIdType, self).__init__(*args, **kwargs)

    def get_model(self):
        """Return model, imports it the first time if attribute is a string."""
        if isinstance(self.model, basestring):
            app_label, name = self.model.split('.', 1)
            self.model = get_model(app_label, name)
        return self.model

    def as_field(self, *args, **kwargs):
        """Prepares form field, uses custom url if preset or tries to build from
        model class. Raises AttributeError if none of this fields are defined"""
        if self.url:  # try url
            url = self.url
        elif self.model:  # try building using model options
            model = self.get_model()
            opt = model._meta
            url = reverse('admin:%s_%s_changelist' % (opt.app_label,
                                                      opt.module_name))
        else:
            raise AttributeError, 'model or url must be defined'
        return self.field(widget=RelatedWidget(url), *args, **kwargs)

    def form_value(self, value):
        """Return raw value to be used in form"""
        return value

    def value(self, value):
        """Returns model instance from value, value will be iterpreted as
        a model id. Returns none if object does not exist."""
        model = self.get_model()
        qs = model.objects.get_query_set()
        if self.select_related:
            qs = qs.select_related(*self.select_related)
        try:
            return qs.get(**{self.model_key: value})
        except model.DoesNotExist:
            return None

    def rawid_context(self, request):
        """Custom rawid index. If a custom rawid list is needed
        (like wsmodels/Product case), this method will be invoked requesting
        context values to be displayed by cms/rawid.html template.
        """
        raise NotImplementedError, 'implement in subclass'


class Text(PlainType):
    """Simple text type"""
    description = 'Text (one line)'


class BigText(PlainType):
    """Rich text type"""
    description = 'Large text (multiple lines)'
    field = RichTexareaField


class Option(PlainType):
    """Defined options data type"""
    description = 'Options (combo select)'
    field = ChoiceField

    def __init__(self, choices):
        """Init method.
        @choices is a list of pairs that will be used when creating
        form field and validate on saving.
        """
        self.choices = choices

    def as_field(self, *args, **kwargs):
        """Prepares form field, uses choices"""
        kwargs.pop('choices', None) # remove any possible choices
        return self.field(choices=self.choices, *args, **kwargs)

    def to_database(self, form, name):
        """Prepares value to database storage, raises ValueError if value is
        missing"""
        value = super(Option, self).to_database(form, name)
        if value not in dict(self.choices):
            raise ValueError, '"%s" is not a valid choice' % value
        return value


class Flag(PlainType):
    """True/False data type"""
    description = 'On/Off flag'
    field = BooleanField

    def value(self, value):
        """Interprets value using django BooleanField way, this will
        parse t, True or 1 as True values and f, False, 0 as False ones."""
        return models.BooleanField().to_python(value)


class Image(PlainType):
    """Image data type"""
    description = 'Image'
    field = PreviewImageField # use custom field

    def to_database(self, form, name):
        """"Saves image data to filesystem and returns file generated name
        to be stored in database"""
        if name not in form.files: # no file uploaded
            raise ValueError, 'missing value'
        data = form.files[name]
        obj = self._model() # hackish way to save file
        obj.value.save(data.name, data, save=False) # use field value to save
        return obj.value.name

    def _model(self, value=None):
        """Returns a django model instance with a value field of ImageField type,
        handy to save or read image file. @value will be loaded as image value."""
        class _Model(models.Model):
            value = models.ImageField(upload_to=IMAGES_UPLOAD_TO, blank=True)
        return _Model(value=value)

    def to_xml(self, value):
        """Convert image data to be stored in a XML file, the image is enconded
        in base64. Image name and enconded value are returned."""
        img = image_to_b64(self._model(value).value)
        if img is not None:
            name, value = img
            return {'file_name': name, 'value': value}
        else:
            return {'value': value}

    def from_xml(self, data):
        """Reads image stored in a XML file, value must be an base64 enconded
        image and name must be present."""
        if 'value' in data and 'file_name' in data:
            return save_b64_image(data['value'], data['file_name'],
                                  self._model().value)

    def value(self, value):
        """Return image value as a model would return it"""
        img = self._model(value or None).value
        if img.name:
            return img


class Date(PlainType):
    """Date data type"""
    description = 'Date'
    field = AdminDateField

    def value(self, value):
        """Return date instance"""
        return models.DateField().to_python(value)


class DateTime(PlainType):
    """Datetime data type"""
    description = 'DateTime'
    field = AdminDateTimeField

    def value(self, value):
        """Return date instance"""
        return models.DateTimeField().to_python(value)


# Base types definition
BASE_TYPES = [Text, BigText, Image, Option, Flag, Date, DateTime]

## Define your project related types here, example:
#
# from django.contrib.auth.models import User
# 
# from tcms.data_types import RawIdType
# 
# __all__ = ['User', 'LOCAL_TYPES']
# 
# 
# class User(RawIdType):
#     """Django auth user data type"""
#     description = 'Django Auth User'
#     model = User
# 
# # Types definition
# LOCAL_TYPES = [User]



# -*- coding: utf-8 -*-
from operator import add

from django.template import Library

from tcms.models import Path, WIP, LIVE


register = Library()


@register.inclusion_tag('cms/template_filter.html')
def admin_template_filter(cl, spec):
    return {'title': spec.title(),
            'choices' : list(spec.choices(cl))}


@register.filter
def similar_pages(page):
    if page:
        return reduce(add, (list(path.pages.filter(state__in=(WIP, LIVE)))
                             for path in Path.objects.filter(path=page.path.path)))
    else:
        return []



# -*- coding: utf-8 -*-
"""Setup file for easy installation"""
from os.path import join, dirname
from setuptools import setup


version = __import__('tcms').__version__

LONG_DESCRIPTION = """
Django tCMS an easy to setup CMS that integrates easily with django-admin.

django-tcms was brought to life by Matías Aguirre while hacking a CMS framework
for Mydeco while working on Insophia.
"""

def long_description():
    """Return long description from README.rst if it's present
    because it doesn't get installed."""
    try:
        return open(join(dirname(__file__), 'README.rst')).read()
    except IOError:
        return LONG_DESCRIPTION


setup(name='django-tcms',
      version=version,
      author=u'Matías Aguirre',
      author_email='matiasaguirre@gmail.com',
      description='Django tCMS.',
      license='BSD',
      keywords='django, cms, django-admin',
      url='https://github.com/omab/django-tcms',
      packages=['tcms',
                'tcms.templatetags'],
      package_data={
          'tcms': [
              'templates/cms/*.html',
              'templates/cms/edit/*.html',
              'static/css/*.css',
              'static/js/*.js',
              'static/img/*.png',
              'static/img/*.gif',
              'static/img/*.gif',
              'static/img/fancybox/*.gif',
              'static/img/fancybox/*.png',
          ]
      },
      long_description=long_description(),
      install_requires=['django>=1.2.5'],
      classifiers=['Framework :: Django',
                   'Development Status :: 4 - Beta',
                   'Topic :: Internet',
                   'License :: OSI Approved :: BSD License',
                   'Intended Audience :: Developers',
                   'Environment :: Web Environment',
                   'Programming Language :: Python :: 2.5',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7'])



# -*- coding: utf-8 -*-
from urlparse import urljoin
from xml.etree import ElementTree
from operator import or_

from django.db import models, transaction
from django.db.models.query import Q
from django.conf import settings
from django.utils.xmlutils import SimplerXMLGenerator
from django.utils.safestring import mark_safe
from django.utils.importlib import import_module
from django.core.exceptions import ValidationError

from tcms.data_types import BASE_TYPES
from tcms.tpl import PAGES, RENDER_EXTRA_CONTEXT
from tcms.utils import save_b64_image, image_to_b64, update_cache, \
                       normalize_path, dotted_dict_to_choices


# page states
WIP, LIVE, OLD = ('wip', 'live', 'old')
STATES = ((WIP, 'Work in Progress'), (LIVE, 'Live'), (OLD, 'Old page'))

# Load extra data types
EXTRA_TYPES_SETTINGS = getattr(settings, 'EXTRA_TYPES_SETTINGS', None)
if EXTRA_TYPES_SETTINGS:
    try:
        module, name = EXTRA_TYPES_SETTINGS.rsplit('.', 1)
        LOCAL_TYPES = getattr(import_module(module), name, [])
    except (AttributeError, ImportError):
        LOCAL_TYPES = []
else:
    LOCAL_TYPES = []

TYPES_MAP = dict((Type.__name__.lower(), Type)
                    for Type in BASE_TYPES + LOCAL_TYPES)

CMSID = 'cmsid'
IMAGES_UPLOAD_TO = getattr(settings, 'TCMS_IMAGES_UPLOAD_TO',
                           'cms/image/%Y/%m/%d')


class Path(models.Model):
    """A CMS Page path."""
    path = models.CharField(max_length=200)
    locale = models.CharField(max_length=255, blank=True, default='',
                              choices=settings.LANGUAGES)

    def save(self, *args, **kwargs):
        """Save method. Enforces absolute paths."""
        self.path = normalize_path(self.path)
        super(Path, self).save(*args, **kwargs)

    def full_url(self):
        """Returns path full url"""
        return urljoin(settings.SITE_URL, self.path)

    def live(self):
        """Returns current live Page for current instance path and
        locale values."""
        try:
            return self.pages.get(state=LIVE)
        except Page.DoesNotExist:
            pass

    def __unicode__(self):
        return self.path

    def __str__(self):
        return self.path

    class Meta:
        unique_together = ('path', 'locale')
        verbose_name = 'URL'


class Page(models.Model):
    """A CMS Page"""
    path = models.ForeignKey(Path, related_name='pages', verbose_name='URL')
    template = models.CharField(max_length=512,
                                choices=dotted_dict_to_choices(PAGES))
    state = models.CharField(max_length=20, default=WIP, choices=STATES)
    description = models.TextField(blank=True)
    updated = models.DateTimeField(editable=False, auto_now_add=True)

    # metadata
    meta_title = models.CharField(max_length=1024, blank=True, default='',
                                  help_text='Metadata title content')
    meta_description = models.CharField(max_length=1024, blank=True, default='',
                                    help_text='Metadata description content')
    meta_keywords = models.CharField(max_length=1024, blank=True, default='',
                                     help_text='Metadata keywords content')

    # values for search indexers
    search_image = models.ImageField(upload_to=IMAGES_UPLOAD_TO, blank=True,
                                     help_text='Image shown on search results')
    search_text = models.TextField(blank=True, default='',
                           help_text='Description displayed on search results')

    def __init__(self, *args, **kwargs):
        super(Page, self).__init__(*args, **kwargs)
        self._loaded = False
        self._rendered = False
        self._template = None

    @property
    def thumbnail(self):
        """Return @search_image as page thumbnail if it's needed to be included
        it's usually used on sharing sites like facebook"""
        return self.search_image

    def save(self, *args, **kwargs):
        """Save handler, will ensure that only one WIP exists per path"""
        if not self.id:
            Page.validate_unique_wip(self.path)
        super(Page, self).save(*args, **kwargs)

    @classmethod
    def validate_unique_wip(cls, path):
        """Validates WIP state uniqueness for path @path"""
        if Page.objects.filter(path=path, state=WIP).count() > 0:
            raise ValidationError(
                    'Another "Work in progress" page exist for given URL')

    @property
    def preview_url(self):
        return '%s?%s=%s&cms_locale=%s' % (self.path.path, CMSID, self.pk,
                                           self.path.locale)

    @property
    def tpl(self):
        """Returns Template Page instance for current defined template"""
        if self._template is None:
            self._template = PAGES.get(self.template)()
        return self._template

    def load(self, sections=None, rendered=False):
        """
        Load page data.
            If @section is passed, only data for that section is loaded.
            If @rendered flag is passed, only rendered data is loaded
        """
        if not self._loaded:
            if sections:
                if not isinstance(sections, (list, tuple)):
                    sections = (sections,)
                sections = reduce(or_, (Q(name__startswith=section)
                                            for section in sections))

            self._rendered = rendered
            if rendered: # load rendered content
                qs = self.rendered_data.values_list('name', 'value')
                if sections:
                    qs = qs.filter(sections)
                for name, value in qs:
                    setattr(self, name, mark_safe(value))
            else: # load values
                qs = self.values.values_list('name', 'value')
                if sections:
                    qs = qs.filter(sections)
                self.tpl.load(qs)
            self._loaded = True

    @property
    def is_live(self):
        """Return True if page is in Live state or False in other case"""
        return self.state == LIVE

    def refresh(self, *args, **kwargs):
        """Refresh rendered data, subsections are rendered by page template
        sections and stored. Extra arguments will be passed to section
        rendering method.
        """
        extra_context = kwargs.pop('extra_context', {})

        if RENDER_EXTRA_CONTEXT:
            extra_context.update(RENDER_EXTRA_CONTEXT)

        extra_context['cms'] = self
        kwargs['extra_context'] = extra_context

        self.load()
        for name, value in self.tpl.render_sections(*args, **kwargs):
            obj, created = Rendered.objects.get_or_create(page=self, name=name)
            obj.value = value
            obj.save()

    @transaction.commit_on_success
    def publish(self, *args, **kwargs):
        """Publish page, will generate rendered content and unpublish current
        live page with same path and state.

        Arguments will be passed to refresh method which will be passed to
        rendering method.
        """
        similar = {'path': self.path, 'state': LIVE}
        for page in Page.objects.filter(**similar):
            page.unpublish()

        self.refresh(*args, **kwargs)
        self.state = LIVE
        self.save()
        update_cache()

    def unpublish(self):
        """Unpublish page, rendered content is not droped"""
        self.state = OLD
        self.save()
        update_cache()

    @transaction.commit_on_success
    def delete(self):
        """Delete a Page, only no Live pages can be deleted. Rendered content
        and values are deleted too"""
        if self.state == LIVE:
            raise TypeError('live pages cannot be deleted')
        self.rendered_data.all().delete()
        self.values.all().delete()
        super(Page, self).delete()
        update_cache()

    @transaction.commit_on_success
    def copy(self, path):
        """Return copy of this page.

        Copies current page under a new @path. This method behaves like
        cloning if path is the same as current one.
        """
        args = {'path': path,
                'template': self.template,
                'description': self.description,
                'meta_title': self.meta_title,
                'meta_description': self.meta_description,
                'meta_keywords': self.meta_keywords,
                'search_image': self.search_image,
                'search_text': self.search_text}
        copy = Page(**args)
        copy.save()

        # duplicate values
        for value in self.values.all():
            value.id = None
            copy.values.add(value)

        return copy

    def to_xml(self, out, encoding=settings.DEFAULT_CHARSET):
        """Exports page data in a XML formated file. It stores
        page info as first item and value data following it.
        Files are base64 encoded and exported too.

        Example:
        <cms-page>
          <page path="/path/" template="homepage" description="Homepage"
                locale="en-gb" meta_title="Title" meta_description="Description"
                meta_keywords="Keywords" search_image="base64 encoded image"
                search_image_name="file name of search image"
                search_text="Search text"></page>
          <value name="heading" type="text" value="Page title"></value>
          <value name="an_image" type="image" value="bas64 enconded"></value>
          ...
        </cms-page>
        """
        xml = SimplerXMLGenerator(out=out, encoding=encoding)
        xml.startDocument()
        xml.startElement('cms-page', {})

        # store page info in first child
        data = {'path': self.path.path, 'template': self.template,
                'description': self.description, 'locale': self.path.locale,
                'meta_title': self.meta_title,
                'meta_description': self.meta_description,
                'meta_keywords': self.meta_keywords,
                'search_text': self.search_text}
        if self.search_image: # add image content if any
            img = image_to_b64(self.search_image)
            if img is not None:
                name, content = img
                data['search_image_name'] = name
                data['search_image'] = content

        xml.startElement('page', data)
        xml.endElement('page')

        # store values
        values = self.values.values_list('name', 'type', 'value')
        for name, value_type, value in values:
            attrs = {'name': name, 'type': value_type, 'value': value}

            if value_type in TYPES_MAP:
                attrs.update(TYPES_MAP[value_type]().to_xml(value))
            xml.startElement('value', attrs)
            xml.endElement('value')

        xml.endElement('cms-page')
        xml.endDocument()
        return out

    @classmethod
    def valid_xml(cls, source):
        """Validatos source as a valid XML exported page"""
        # check XML structure
        try:
            etree = ElementTree.fromstring(source)
        except Exception, e:
            raise ValidationError('Malformed XML or not an XML document "%s"' \
                                        % str(e))

        # check page information (it's stored in first subnode)
        page_info = etree[0].attrib

        # check template
        tpl = page_info.get('template')
        if not tpl or tpl not in PAGES:
            raise ValidationError('Malformed XML: wrong template %s' % tpl)

        # check path
        path = page_info.get('path')
        if not path:
            raise ValidationError('Malformed XML: missing path')

        # check path/locale pair, if it does't exists importing
        # is safe because will be created, if it exists, the page
        # state needs to be checked because only one WIP page per
        # path is allowed
        locale = page_info.get('locale', '')
        try:
            path = Path.objects.get(path=path, locale=locale)
        except Path.DoesNotExist:
            pass
        else:
            Page.validate_unique_wip(path)

    @classmethod
    def from_xml(cls, source):
        """Import a page information. @source must be an XML exported
        valid page.
        """
        # page *must* be validated first
        etree = ElementTree.fromstring(source) # parse XML

        page_info = etree[0].attrib # page info in first child

        path = page_info.get('path')
        locale = page_info.get('locale', '')
        path, created = Path.objects.get_or_create(path=path, locale=locale)

        # save page data, image is saved later if any
        data = {'meta_title': page_info.get('meta_title', ''),
                'meta_description': page_info.get('meta_description', ''),
                'meta_keywords': page_info.get('meta_keywords', ''),
                'search_text': page_info.get('search_text', ''),
                'description': page_info.get('description', ''),
                'template': page_info.get('template'), 'path': path}
        page = Page(**data)
        page.save()

        # save search image if any
        save_b64_image(page_info.get('search_image'),
                       page_info.get('search_image_name'),
                       page.search_image, save=True)

        # values are from second child to end
        for node in etree[1:]:
            attr = node.attrib
            name, value_type, value = attr['name'], attr['type'], attr['value']
            if value_type in TYPES_MAP:
                value = TYPES_MAP[value_type]().from_xml(attr)
            page.values.add(Value(name=name, value=value, type=value_type))
        return page

    def __getitem__(self, name):
        """Get item handler, once page data is loaded is possible to
        access page section using [] notation.
        """
        if self._loaded and self._rendered and hasattr(self, name):
            # return rendered content if available
            return mark_safe(getattr(self, name, '').strip())
        return self.tpl[name] # return page section

    def __unicode__(self):
        return unicode(self.path)

    def __str__(self):
        return str(self.path)


class Value(models.Model):
    """CMS Page value
    @page  is page the value belongs too
    @name  is used to generate values namspaces. First token denotes section
           that value belongs too, second name denotes section fragment and
           follow up names describe grouping and list position. Last token is
           value name
    @value is store database value
    @type  is data type name
    """
    page = models.ForeignKey(Page, related_name='values')
    name = models.CharField(max_length=255)
    value = models.TextField(blank=True)
    type = models.CharField(max_length=32, choices=((name, Type.description)
                                    for name, Type in TYPES_MAP.iteritems()))

    def __unicode__(self):
        return u'%s (%s)' % (self.name, self.value[:20])

    class Meta:
        unique_together = ('page', 'name')


class Rendered(models.Model):
    """CMS Page rendered content for rapid page loading"""
    page = models.ForeignKey(Page, related_name='rendered_data')
    name = models.CharField(max_length=64, blank=False)
    value = models.TextField()

    class Meta:
        unique_together = ('page', 'name')
